use modules::Module;
use toml::Table;

use std::sync::Arc;
use std::sync::mpsc::{Sender, Receiver};

use helpers::command::split_command;
use event::{Event, Envelope};
use rand;

pub struct RandomModule {
    prefix: String,
    responses: Vec<String>
}

impl RandomModule {
    pub fn new (configuration: &Table) -> Box<Module> {
        let prefix = configuration.get("prefix")
                                  .and_then(|value| value.as_str())
                                  .unwrap_or("!random ");

        let responses = configuration.get("responses")
                                     .and_then(|value| value.as_slice())
                                     .map(|value| value.to_vec())
                                     .unwrap_or(vec![])
                                     .into_iter()
                                     .map(|value| { String::from(value.as_str().unwrap()) })
                                     .collect();

        Box::new(RandomModule {
            prefix: String::from(prefix),
            responses: responses
        })
    }
}

impl Module for RandomModule {
    fn run(&self, _: Sender<Envelope>, receiver: Receiver<Arc<Envelope>>) {
        let mut rng = rand::thread_rng();

        loop {
            match receiver.recv() {
                Ok(envelope) => {
                    match *envelope.event {
                        Event::Message { ref message } => {
                            debug!("Received message... {:?}", message.content);
                            match split_command(&message.content) {
                                Some((command, _)) => {
                                    if command == self.prefix {
                                        let response = &rand::sample(&mut rng, &self.responses, 1)[0][..];
                                        message.reply(response);
                                    }
                                },
                                _ => {}
                            }
                        }
                        _ => ()
                    }
                }
                Err(error) => { error!("Error {:?}", error) }
            }
        }
    }
}
